==> kari-0.30.tar.gz: kari-0.30/chloe/header.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "utils.h"
#include "header.h"

void skip_line (const char *buffer, const int buffer_length, int *counter);

int
parse_header (const char *header, const int header_length, char **body, int *body_length, int *body_counter)
{

  int i = 0;
  int j;

  *body_length = -1;


  while (i < header_length)
    {
      if (!(strncasecmp(header+i, "Content-Length: ", 16)))
	{
	  i += 16;
	  *body_length = atoi (header+i);
	  debug (debug_chloe, "parse_header: Content-Length is %d", *body_length);
	}
      if (!(strncmp(header+i, "\r\n", 2)))
	{
	  i += 2;
	  debug (debug_chloe, "parse_header: header ran out at %d/%d; copying rest to buffer", 
		 i, header_length);
	  *body = xmalloc (*body_length + 1);
	  *body_counter = 0;
	  j = 0;
	  while (i < header_length && j < *body_length)
	    {
	      (*body)[j++] = header[i++];
	      (*body_counter)++;
	    }
	  break;
	}

      skip_line (header, header_length, &i);
    }

  if (*body_length == -1)
    fubar (NONSYS, "Ran out of header, but we don't know the content-length; abuse James, this should be handled");

  return 0; /* FIXME: should be status code */

}
     
void skip_line (const char *buffer, const int buffer_length, int *counter)
{

  while (*counter < buffer_length && buffer[*counter] != '\n')
    (*counter)++;

  (*counter)++;          /* Skip the newline */

}

==> kari-0.32.tar.gz: kari-0.32/chloe/header.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "utils.h"
#include "header.h"

void skip_line (const char *buffer, const int buffer_length, int *counter);

int
parse_header (const char *header, const int header_length, int fd, int *body_length)
{

  int i = 0;
  int amount_written;

  *body_length = -1;

  while (i < header_length)
    {
      if (!(strncasecmp(header+i, "Content-Length: ", 16)))
	{
	  i += 16;
	  *body_length = atoi (header+i);
	  debug (debug_chloe, "parse_header: Content-Length is %d", *body_length);
	}
      if (!(strncmp(header+i, "\r\n", 2)))
	{
	  i += 2;
	  debug (debug_chloe, "parse_header: header ran out at %d/%d; copying rest to buffer", 
		 i, header_length);
	  
	  amount_written = write (fd, header+i, header_length - i);
	  if (amount_written != header_length - i)
	    fubar (SYSERR, "parse_header: couldn't dump rest of body from header-get run.");
	  if (*body_length != -1)
	    (*body_length) -= (header_length - i);

	  break;
	}

      skip_line (header, header_length, &i);
    }

  return 0; /* FIXME: should be status code */

}
     
void skip_line (const char *buffer, const int buffer_length, int *counter)
{

  while (*counter < buffer_length && buffer[*counter] != '\n')
    (*counter)++;

  (*counter)++;          /* Skip the newline */

}
